import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Random;

public class ClientHandlerThread extends Thread {
    private final Socket socket;
    private final BufferedReader from_client;
    private final PrintWriter to_client;
    private static int num_instances = 0;
    private int instance_number = ++num_instances;
    private final SimpleLogger logger = new SimpleLogger("ClientHandlerThread " + instance_number);
	private static final SimpleDateFormat date_format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS Z");
	private final Random random = new Random();

	public ClientHandlerThread(Socket socket, BufferedReader from_client, PrintWriter to_client) {
		this.socket = socket;
        this.from_client = from_client;
        this.to_client = to_client;
	}
	
	public void run() {
		Date now = new Date();
		String formatted_date = date_format.format(now);
		String hello_line = "DistributedCalculationsServer " + DistributedCalculationsServer.VERSION;
		to_client.println(hello_line);
		try {
			String line_from_client = from_client.readLine();
			if (line_from_client == null) {
				logger.log("Got line from client: null");
			}
			else {
				logger.log("Got line from client: '" + line_from_client + "'");
				int length = line_from_client.length();
				int position = 0;
				while (position < length && Character.isWhitespace(line_from_client.charAt(position)))
					++position;
				int command_start = position;
				// while (position < length && Character.isLetter(line_from_client.charAt(position)))
				while (position < length && !Character.isWhitespace(line_from_client.charAt(position)))
					++position;
				String command = line_from_client.substring(command_start, position);
				while (position < length && Character.isWhitespace(line_from_client.charAt(position)))
					++position;
				int arguments_start = position;
				
				if (command.equals("GET")) {
					// Here we should check that there are no extra arguments
					String reply = "SOLVE " + random.nextInt(Integer.MAX_VALUE) + " " + 1000 * random.nextDouble() + " " + getRandomOp() + " " + 1000 * random.nextDouble(); 
					to_client.println(reply);
				}
				else if (command.equals("SOLVED")) {
					// Here we should check and handle the arguments
					String reply = "THANKS";
					to_client.println(reply);
				}
				else {
					String reply = "ERROR";
					to_client.println(reply);
				}
			}
			to_client.close();
			from_client.close();
			socket.close();
		} catch (IOException e) {
			logger.log("IOException (" + e + ") when closing from_client");
		}
	} // run

	private String getRandomOp() {
		String op = "ERROR";
		switch (random.nextInt(4)) {
		case 0: op = "+"; break;
		case 1: op = "-"; break;
		case 2: op = "*"; break;
		case 3: op = "/"; break;
		default: op = "ERROR"; break;
		}
		return op;
	}
} // class ClientHandlerThread
